import 'dart:async';

import 'package:flame/components.dart';
import 'package:flame/extensions.dart';
import 'package:flame/game.dart';
import 'package:flame/palette.dart';
import 'package:flame_spatial_grid/flame_spatial_grid.dart';
import 'package:flutter/material.dart' show Colors, EdgeInsets;

class MinimalGame extends FlameGame with HasSpatialGridFramework {
  MinimalGame();

  final world = MyWorld();
  late final CameraComponent cameraComponent;
  late final JoystickComponent joystick;

  @override
  FutureOr<void> onLoad() async {
    final knobPaint = BasicPalette.lightBlue.paint();
    final backgroundPaint = BasicPalette.blue.paint();

    joystick = JoystickComponent(
      knob: CircleComponent(radius: 25, paint: knobPaint),
      background: CircleComponent(radius: 100, paint: backgroundPaint),
      margin: const EdgeInsets.only(left: 40, bottom: 40),
      position: Vector2.all(32),
    );

    final player = Player(position: Vector2(160, 190));

    cameraComponent = CameraComponent(
      world: world,
      hudComponents: [joystick], // <- Uncomment
    );
    cameraComponent.viewfinder.zoom = 1;
    cameraComponent.priority = 999;
    cameraComponent.follow(player, maxSpeed: 200, snap: true);

    await initializeSpatialGrid(
      cellSize: 50,
      rootComponent: world,
      trackedComponent: SpatialGridDebugCameraWrapper(cameraComponent),
      trackWindowSize: false,
      debug: true,
    );

    add(world);
    world.add(player);
    return super.onLoad();
  }
}

class SpatialGridDebugCameraWrapper extends SpatialGridCameraWrapper {
  SpatialGridDebugCameraWrapper(super.cameraComponent);

  @override
  void onAfterZoom() {
    try {
      if (!game.isSpatialGridDebugEnabled) {
        game.onAfterZoom();
      }
      // ignore: avoid_catches_without_on_clauses, empty_catches
    } catch (e) {}
  }
}

class Player extends PositionComponent with HasGridSupport, HasPaint {
  Player({super.position}) : super(size: Vector2(10, 10)) {
    paint.color = Colors.indigoAccent;
    debugMode = true;
  }

  late final _rect = Rect.fromLTWH(0, 0, size.x, size.y);

  @override
  void render(Canvas canvas) {
    canvas.drawRect(_rect, paint);
  }
}

class MyWorld extends World with HasGameRef<MinimalGame> {}
